unit uGrid;

interface

uses
  System.SysUtils, System.Classes, JS, WEBLib.JSON, Web, WEBLib.Graphics, WEBLib.Controls,
  WEBLib.Forms, WEBLib.Dialogs, WEBLib.ExtCtrls, Vcl.Controls, WEBLib.TreeNodes,
  Vcl.ComCtrls, WEBLib.ComCtrls, WEBLib.ContinuousScroll, Vcl.StdCtrls,
  WEBLib.StdCtrls, jsdelphisystem, Data.DB, WEBLib.CDS, WEBLib.DBCtrls,
  WEBLib.DB, WEBLib.Grids, Vcl.Grids, uDetail;

type
  TfrmGrid = class(TfrmDetail)
    pNav: TWebPanel;
    navMain: TWebDBNavigator;
    mRowsJSON: TWebMemo;
    grdMain: TWebDBGrid;
    WebSplitter1: TWebSplitter;
    mLog: TWebMemo;
    btnClearLog: TWebButton;
    btnProblem1: TWebButton;
    btnFocusGrid: TWebButton;
    procedure btnClearLogClick(Sender: TObject);
    procedure btnProblem1Click(Sender: TObject);
    procedure grdMainKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnFocusGridClick(Sender: TObject);
  private
    { Private declarations }
    cdMain: TWebClientDataset;
    dsMain: TWebDataSource;
    procedure Log(AValue: String; AddInfo: Boolean = False);
    procedure DatasetBeforeOpen(DataSet: TDataSet);
    procedure DatasetAfterOpen(DataSet: TDataSet);
    procedure DatasetBeforeInsert(DataSet: TDataSet);
    procedure DatasetAfterInsert(DataSet: TDataSet);
    procedure DatasetOnNewRecord(DataSet: TDataSet);
    procedure DatasetBeforeEdit(DataSet: TDataSet);
    procedure DatasetAfterEdit(DataSet: TDataSet);
    procedure DatasetBeforeScroll(DataSet: TDataSet);
    procedure DatasetAfterScroll(DataSet: TDataSet);
    procedure DatasetBeforePost(DataSet: TDataSet);
    procedure DatasetAfterPost(DataSet: TDataSet);
    procedure DatasetBeforeDelete(DataSet: TDataSet);
    procedure DatasetAfterDelete(DataSet: TDataSet);
    procedure DatasetBeforeCancel(DataSet: TDataSet);
    procedure DatasetAfterCancel(DataSet: TDataSet);
    procedure SimulateKey(AKey: String);
  public
    { Public declarations }
    procedure SetupForm(); override;
  end;

implementation

{$R *.dfm}

procedure TfrmGrid.SetupForm();
var
  LFieldDefs: TFieldDefs;
  LFieldDef: TFieldDef;
  S: String;
  I: Integer;
  LColumn: TWebGridColumn;
begin
  inherited SetupForm();

  cdMain := TWebClientDataSet.Create(nil);
  cdMain.BeforeOpen := DatasetBeforeOpen;
  cdMain.AfterOpen := DatasetAfterOpen;
  cdMain.BeforeInsert := DatasetBeforeInsert;
  cdMain.AfterInsert := DatasetAfterInsert;
  cdMain.OnNewRecord := DatasetOnNewRecord;
  cdMain.BeforeEdit := DatasetBeforeEdit;
  cdMain.AfterEdit := DatasetAfterEdit;
  cdMain.BeforeScroll := DatasetBeforeScroll;
  cdMain.AfterScroll := DatasetAfterScroll;
  cdMain.BeforePost := DatasetBeforePost;
  cdMain.AfterPost := DatasetAfterPost;
  cdMain.BeforeDelete := DatasetBeforeDelete;
  cdMain.AfterDelete := DatasetAfterDelete;
  cdMain.BeforeCancel := DatasetBeforeCancel;
  cdMain.AfterCancel := DatasetAfterCancel;

  LFieldDefs := TFieldDefs.Create(nil);

  LFieldDef := LFieldDefs.AddFieldDef();
  LFieldDef.DataType := ftInteger;
  LFieldDef.DisplayName := 'Id';
  LFieldDef.Name := 'id';

  LFieldDef := LFieldDefs.AddFieldDef();
  LFieldDef.DataType := ftString;
  LFieldDef.DisplayName := 'Code';
  LFieldDef.Name := 'code';

  LFieldDef := LFieldDefs.AddFieldDef();
  LFieldDef.DataType := ftMemo;
  LFieldDef.DisplayName := 'Description';
  LFieldDef.Name := 'descr';

  LFieldDef := LFieldDefs.AddFieldDef();
  LFieldDef.DataType := ftFloat;
  LFieldDef.DisplayName := 'Amount';
  LFieldDef.Name := 'amount';

  LFieldDef := LFieldDefs.AddFieldDef();
  LFieldDef.DataType := ftDateTime;
  LFieldDef.DisplayName := 'Timestamp';
  LFieldDef.Name := 'ts';

  cdMain.FieldDefs := LFieldDefs;

  Log('cdMain.Rows');
  cdMain.Rows := TJSArray(TJSJSON.parseObject(mRowsJSON.Lines.Text));

  Log('cdMain.Open');
  cdMain.Open;

  grdMain.Columns.Clear;

  LColumn := grdMain.Columns.Add;
  LColumn.ElementClassName := '';
  LColumn.DataField := 'id';
  LColumn.DataType := TColumnDataType.cdText;
  LColumn.Width := 200;
  LColumn.Title := 'Id';
  LColumn.TitleElementClassName := '';

  LColumn := grdMain.Columns.Add;
  LColumn.ElementClassName := '';
  LColumn.DataField := 'code';
  LColumn.DataType := TColumnDataType.cdText;
  LColumn.Width := 200;
  LColumn.Title := 'Code';
  LColumn.TitleElementClassName := '';

  LColumn := grdMain.Columns.Add;
  LColumn.ElementClassName := '';
  LColumn.DataField := 'descr';
  LColumn.DataType := TColumnDataType.cdText;
  LColumn.Width := 200;
  LColumn.Title := 'Description';
  LColumn.TitleElementClassName := '';

  LColumn := grdMain.Columns.Add;
  LColumn.ElementClassName := '';
  LColumn.DataField := 'amount';
  LColumn.DataType := TColumnDataType.cdText;
  LColumn.Width := 200;
  LColumn.Title := 'Amount';
  LColumn.TitleElementClassName := '';

  LColumn := grdMain.Columns.Add;
  LColumn.ElementClassName := '';
  LColumn.DataField := 'ts';
  LColumn.DataType := TColumnDataType.cdText;
  LColumn.Width := 200;
  LColumn.Title := 'Timestamp';
  LColumn.TitleElementClassName := '';

  Log('DataSource.DataSet := cdMain');
  DataSource.DataSet := cdMain;

  Log('grdMain.DataSource := DataSource');
  grdMain.DataSource := DataSource;

  Log('navMain.DataSource := DataSource');
  navMain.DataSource := DataSource;

  btnProblem1Click(nil);
end;

procedure TfrmGrid.Log(AValue: String; AddInfo: Boolean = False);
var
  DataSet: TDataSet;
begin
  mLog.Lines.Add(AValue);
  if (AddInfo) and (cdMain<>nil) and (cdMain.Active) then
  begin
    DataSet := TDataSet(cdMain);
    mLog.Lines.Add('   State: ' + IntToStr(Integer(DataSet.State)));
    mLog.Lines.Add('   RecordCount: ' + IntToStr(DataSet.RecordCount));
    mLog.Lines.Add('   RecNo: ' + IntToStr(DataSet.RecNo));
  end;
end;

procedure TfrmGrid.btnClearLogClick(Sender: TObject);
begin
  inherited;
  mLog.Lines.Clear;
end;

procedure TfrmGrid.DatasetBeforeOpen(DataSet: TDataSet);
begin
  Log('DatasetBeforeOpen', True);
end;

procedure TfrmGrid.DatasetAfterOpen(DataSet: TDataSet);
begin
  Log('DatasetAfterOpen', True);
end;

procedure TfrmGrid.DatasetBeforeInsert(DataSet: TDataSet);
begin
  Log('DatasetBeforeInsert', True);
end;

procedure TfrmGrid.DatasetAfterInsert(DataSet: TDataSet);
begin
  Log('DatasetAfterInsert', True);
end;

procedure TfrmGrid.DatasetOnNewRecord(DataSet: TDataSet);
begin
  Log('DatasetOnNewRecord', True);
end;

procedure TfrmGrid.DatasetBeforeEdit(DataSet: TDataSet);
begin
  Log('DatasetBeforeEdit', True);
end;

procedure TfrmGrid.DatasetAfterEdit(DataSet: TDataSet);
begin
  Log('DatasetAfterEdit', True);
end;

procedure TfrmGrid.DatasetBeforeScroll(DataSet: TDataSet);
begin
  Log('DatasetBeforeScroll', True);
end;

procedure TfrmGrid.DatasetAfterScroll(DataSet: TDataSet);
begin
  Log('DatasetAfterScroll', True);
end;

procedure TfrmGrid.DatasetBeforePost(DataSet: TDataSet);
begin
  Log('DatasetBeforePost', True);
end;

procedure TfrmGrid.DatasetAfterPost(DataSet: TDataSet);
begin
  Log('DatasetAfterPost', True);
end;

procedure TfrmGrid.DatasetBeforeDelete(DataSet: TDataSet);
begin
  Log('DatasetBeforeDelete', True);
end;

procedure TfrmGrid.DatasetAfterDelete(DataSet: TDataSet);
begin
  Log('DatasetAfterDelete', True);
end;

procedure TfrmGrid.DatasetBeforeCancel(DataSet: TDataSet);
begin
  Log('DatasetBeforeCancel', True);
end;

procedure TfrmGrid.DatasetAfterCancel(DataSet: TDataSet);
begin
  Log('DatasetAfterCancel', True);
end;

procedure TfrmGrid.grdMainKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  Log('grdMainKeyDown Key: ' + IntToStr(Key));
  inherited KeyDown(Key, Shift);
end;

procedure TfrmGrid.SimulateKey(AKey: String);
begin

asm
var event_init = {};
event_init['key'] = AKey;
event_init['code'] = '';
event_init['location'] = 0;
event_init['ctrlKey'] = false;
event_init['shiftKey'] = false;
event_init['altKey'] = false;
event_init['metaKey'] = false;
event_init['repeat'] = false;
event_init['isComposing'] = false;
event_init['charCode'] = 0;
event_init['keyCode'] = 0;
event_init['which'] = 0;
var event = new KeyboardEvent('keydown', event_init);  //keydown, keyup, keypress
var grid = document.getElementById("DBGRIDNormalCellTable").parentElement.parentElement;
grid.dispatchEvent(event);
end;

end;

procedure TfrmGrid.btnFocusGridClick(Sender: TObject);
begin
  inherited;
  if grdMain.CanFocus then
    grdMain.SetFocus;
end;

procedure TfrmGrid.btnProblem1Click(Sender: TObject);
begin
  inherited;
  SimulateKey('ArrowRight');
end;

end.
